<?php

namespace App\Nova\Actions;

use App\Models\User;
use Illuminate\Bus\Queueable;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Support\Collection;
use Laravel\Nova\Actions\Action;
use Laravel\Nova\Actions\ActionResponse;
use Laravel\Nova\Fields\ActionFields;
use Laravel\Nova\Http\Requests\NovaRequest;

class DisableTwoFactorAuthentication extends Action
{
    use InteractsWithQueue;
    use Queueable;

    /**
     * The displayable name of the action.
     *
     * @var string
     */
    public $name = 'تعطيل التحقق الثنائي';

    /**
     * Indicates if this action is destructive.
     *
     * @var bool
     */
    public $destructive = true;

    /**
     * Perform the action on the given models.
     *
     * @return mixed
     */
    public function handle(ActionFields $fields, Collection $models)
    {
        $count = 0;
        foreach ($models as $user) {
            if ($user instanceof User) {
                try {
                    $user->forceFill([
                        'two_factor_secret' => null,
                        'two_factor_recovery_codes' => null,
                        'two_factor_confirmed_at' => null,
                    ])->save();
                    $count++;
                } catch (\Exception $e) {
                    $this->markAsFailed($user, $e);
                }
            }
        }

        if ($count > 0) {
            return Action::message("تم تعطيل التحقق الثنائي لـ {$count} مستخدم.");
        }

        return Action::danger('لم يتم تعطيل التحقق الثنائي لأي مستخدم (ربما حدث خطأ).');
    }

    /**
     * Get the fields available on the action.
     *
     * @return array<int, \Laravel\Nova\Fields\Field>
     */
    public function fields(NovaRequest $request): array
    {
        return [];
    }
}
