<?php

namespace App\Nova;

use Illuminate\Http\Request;
use Laravel\Nova\Fields\ID;
use Laravel\Nova\Fields\Text;
use Laravel\Nova\Fields\BelongsTo;
use Laravel\Nova\Fields\Textarea;
use Laravel\Nova\Fields\Select;
use Laravel\Nova\Fields\MorphMany;
use Laravel\Nova\Fields\Badge;
use Laravel\Nova\Http\Requests\NovaRequest;
use App\Models\Asset as AssetModel;

class Asset extends Resource
{
    /**
     * The model the resource corresponds to.
     *
     * @var class-string<\App\Models\Asset>
     */
    public static $model = \App\Models\Asset::class;

    /**
     * The logical group associated with the resource.
     *
     * @var string
     */
    public static $group = 'إدارة الأصول';

    /**
     * The single value that should be used to represent the resource when being displayed.
     *
     * @var string
     */
    public static $title = 'serial_number';

    /**
     * The columns that should be searched.
     *
     * @var array
     */
    public static $search = [
        'id', 'serial_number',
    ];

    public static function label(): string
    {
        return 'الأصول';
    }

    public static function singularLabel(): string
    {
        return 'أصل';
    }

    /**
     * Get the fields displayed by the resource.
     *
     * @return array<int, \Laravel\Nova\Fields\Field>
     */
    public function fields(NovaRequest $request): array
    {
        return [
            ID::make()->sortable(),

            BelongsTo::make('المدرسة', 'school', School::class)
                ->searchable()
                ->rules('required'),

            BelongsTo::make('نوع الأصل', 'assetCategory', AssetCategory::class)
                // ->searchable()
                ->rules('required'),

            BelongsTo::make('الماركة', 'assetBrand', AssetBrand::class)
                // ->searchable()
                ->rules('required'),

            Text::make('الموديل', 'model_name')
                ->sortable()
                ->rules('required', 'string', 'max:255'),

            Text::make('الرقم التسلسلي', 'serial_number')
                ->sortable()
                ->rules('required', 'string', 'max:255'),

            Select::make('حالة الجهاز', 'condition')
                ->options([
                    'ممتاز' => 'ممتاز',
                    'جيد' => 'جيد',
                    'ضعيف' => 'ضعيف',
                ])
                ->rules('required')
                ->displayUsingLabels()
                ->hideFromIndex()
                ->hideFromDetail(),

            Badge::make('حالة الجهاز', 'condition')
                ->map([
                    'ممتاز' => 'success',
                    'جيد' => 'warning',
                    'ضعيف' => 'danger',
                ])
                ->hideWhenCreating()
                ->hideWhenUpdating(),

            Textarea::make('ملاحظات', 'notes')
                ->nullable(),

            BelongsTo::make('أنشئ بواسطة', 'creator', User::class)
                ->exceptOnForms()
                ->searchable(),

            MorphMany::make('المرفقات', 'attachments', Attachment::class),
        ];
    }

    /**
     * Get the cards available for the resource.
     *
     * @return array<int, \Laravel\Nova\Card>
     */
    public function cards(NovaRequest $request): array
    {
        return [];
    }

    /**
     * Get the filters available for the resource.
     *
     * @return array<int, \Laravel\Nova\Filters\Filter>
     */
    public function filters(NovaRequest $request): array
    {
        return [];
    }

    /**
     * Get the lenses available for the resource.
     *
     * @return array<int, \Laravel\Nova\Lenses\Lens>
     */
    public function lenses(NovaRequest $request): array
    {
        return [];
    }

    /**
     * Get the actions available for the resource.
     *
     * @return array<int, \Laravel\Nova\Actions\Action>
     */
    public function actions(NovaRequest $request): array
    {
        return [];
    }
}
