<?php

namespace App\Nova\Metrics;

use App\Models\Department;
use DateTimeInterface;
use Laravel\Nova\Http\Requests\NovaRequest;
use Laravel\Nova\Metrics\Value;
use Laravel\Nova\Metrics\ValueResult;
use Laravel\Nova\Nova;

class TotalDepartments extends Value
{
    /**
     * The displayable name of the metric.
     *
     * @var string
     */
    public $name = 'إجمالي الإدارات';

    /**
    * Set the default width of the metric card
    *
    * @var string
    */
    public $width = '1/3';

    /**
     * Calculate the value of the metric.
     */
    public function calculate(NovaRequest $request): ValueResult
    {
        // Use the count method that allows comparison (assuming created_at for comparison)
        return $this->count($request, Department::class, null, 'created_at'); 
    }

    /**
     * Get the ranges available for the metric.
     *
     * @return array<int|string, string>
     */
    public function ranges(): array
    {
        // Add ranges for comparison
        return [
            30 => Nova::__('30 يومًا'),
            60 => Nova::__('60 يومًا'),
            365 => Nova::__('365 يومًا'),
            'TODAY' => Nova::__('اليوم'),
            'MTD' => Nova::__('منذ بداية الشهر'),
            'QTD' => Nova::__('منذ بداية الربع السنوي'),
            'YTD' => Nova::__('منذ بداية العام'),
        ];
    }

    /**
     * Determine the amount of time the results of the metric should be cached.
     *
     * @return \DateTimeInterface|\DateInterval|float|int|null
     */
    public function cacheFor(): \DateTimeInterface|\DateInterval|float|int|null
    {
        return now()->addMinutes(5);
    }

    /**
     * Get the URI key for the metric.
     *
     * @return string
     */
    public function uriKey(): string
    {
        return 'total-departments';
    }
}
