<?php

namespace App\Nova\Metrics;

use App\Models\User;
use DateTimeInterface;
use Laravel\Nova\Http\Requests\NovaRequest;
use Laravel\Nova\Metrics\Value;
use Laravel\Nova\Metrics\ValueResult;
use Laravel\Nova\Nova;

class TotalUsers extends Value
{
    /**
     * The displayable name of the metric.
     *
     * @var string
     */
    public $name = 'إجمالي المستخدمين';

    /**
    * Set the default width of the metric card
    *
    * @var string
    */
    public $width = '1/3';

    /**
     * Calculate the value of the metric.
     */
    public function calculate(NovaRequest $request): ValueResult
    {
        // Use the count method that allows comparison
        return $this->count($request, User::class, null, 'created_at'); 
        // Assuming you want to compare based on creation date
    }

    /**
     * Get the ranges available for the metric.
     *
     * @return array<int|string, string>
     */
    public function ranges(): array
    {
        // Add ranges for comparison (e.g., compare to previous 7 days, 30 days)
        return [
            // 7 => Nova::__('7 Days'), // Example: Compare current 7 days vs previous 7 days
            30 => Nova::__('30 يومًا'), // Compare current 30 days vs previous 30 days
            60 => Nova::__('60 يومًا'),
            365 => Nova::__('365 يومًا'),
            'TODAY' => Nova::__('اليوم'), // Compare Today vs Yesterday
            'MTD' => Nova::__('منذ بداية الشهر'), // Compare MTD vs Previous Month To Date
            'QTD' => Nova::__('منذ بداية الربع السنوي'), // Compare QTD vs Previous QTD
            'YTD' => Nova::__('منذ بداية العام'), // Compare YTD vs Previous YTD
            // 'ALL' => Nova::__('All Time'), // Usually no comparison for ALL
        ];
    }

    /**
     * Determine the amount of time the results of the metric should be cached.
     *
     * @return \DateTimeInterface|\DateInterval|float|int|null
     */
    public function cacheFor(): \DateTimeInterface|\DateInterval|float|int|null
    {
        return now()->addMinutes(5);
    }

    /**
     * Get the URI key for the metric.
     *
     * @return string
     */
    public function uriKey(): string
    {
        return 'total-users';
    }
}
