<?php

namespace App\Policies;

use App\Models\Models\Asset;
use App\Models\User;
use Illuminate\Auth\Access\Response;

class AssetPolicy
{
    /**
     * Determine whether the user can view any models.
     */
    public function viewAny(User $user): bool
    {
        return $user->hasPermission('view-any-assets');
    }

    /**
     * Determine whether the user can view the model.
     */
    public function view(User $user, Asset $asset): bool
    {
        return $user->hasPermission('view-assets');
        // Add more specific logic if needed, e.g., based on user's department vs asset's department
    }

    /**
     * Determine whether the user can create models.
     */
    public function create(User $user): bool
    {
        return $user->hasPermission('create-assets');
    }

    /**
     * Determine whether the user can update the model.
     */
    public function update(User $user, Asset $asset): bool
    {
        return $user->hasPermission('update-assets');
        // Add more specific logic if needed
    }

    /**
     * Determine whether the user can delete the model.
     */
    public function delete(User $user, Asset $asset): bool
    {
        return $user->hasPermission('delete-assets');
        // Add more specific logic if needed
    }

    /**
     * Determine whether the user can restore the model.
     */
    public function restore(User $user, Asset $asset): bool
    {
        // Assuming same permission as delete, or create a specific 'restore-assets' permission
        return $user->hasPermission('delete-assets');
    }

    /**
     * Determine whether the user can permanently delete the model.
     */
    public function forceDelete(User $user, Asset $asset): bool
    {
        // Assuming same permission as delete, or create a specific 'force-delete-assets' permission
        return $user->hasPermission('delete-assets');
    }
}
