<?php

use Illuminate\Database\Migrations\Migration;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\Schema;

return new class extends Migration
{
    /**
     * Run the migrations.
     */
    public function up(): void
    {
        Schema::table('schools', function (Blueprint $table) {
            $table->string('location')->after('name')->comment('موقع المدرسة'); // Nullable here, will enforce in Nova
            $table->string('ministerial_number')->unique()->after('location')->comment('الرقم الوزاري');
        });
    }

    /**
     * Reverse the migrations.
     */
    public function down(): void
    {
        Schema::table('schools', function (Blueprint $table) {
            // Check if ministerial_number column exists before dropping
            if (Schema::hasColumn('schools', 'ministerial_number')) {
                 try {
                     // Attempt to drop the unique index (assuming standard name)
                     $table->dropUnique('schools_ministerial_number_unique');
                 } catch (\Exception $e) {
                     // Ignore error if index doesn't exist or dbal is missing
                 }
                // Drop the column
                $table->dropColumn('ministerial_number');
            }
            
            // Check if location column exists before dropping
            if (Schema::hasColumn('schools', 'location')) {
                $table->dropColumn('location');
            }
        });
    }
};
