<?php

use Illuminate\Database\Migrations\Migration;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\Schema;

return new class extends Migration
{
    /**
     * Run the migrations.
     */
    public function up(): void
    {
        Schema::table('assets', function (Blueprint $table) {
            // 1- نوع الأصل (إجباري)
            $table->foreignId('asset_category_id')->constrained('asset_categories')->comment('Asset Category ID');
            
            // 2- الماركة (إجباري)
            $table->foreignId('asset_brand_id')->constrained('asset_brands')->comment('Asset Brand ID');

            // 3- الموديل (إجباري)
            $table->string('model_name')->comment('Model Name/Number');

            // 4- الرقم التسلسلي (إجباري)
            $table->string('serial_number')->comment('Serial Number'); // Consider adding unique() if needed

            // 5- حالة الجهاز (ممتاز - جيد - ضعيف) (إجباري)
            $table->string('condition')->comment('Condition (e.g., excellent, good, poor)');

            // 6- تحميل الصور (مسار الصورة - اختياري)
            $table->string('image_path')->nullable()->comment('Path to the uploaded image');

            // 7- ملاحظات (اختياري)
            $table->text('notes')->nullable()->comment('Additional notes');
        });
    }

    /**
     * Reverse the migrations.
     */
    public function down(): void
    {
        Schema::table('assets', function (Blueprint $table) {
            // Drop foreign keys first to avoid errors
            $table->dropForeign(['asset_category_id']);
            $table->dropForeign(['asset_brand_id']);

            // Drop the columns
            $table->dropColumn([
                'asset_category_id',
                'asset_brand_id',
                'model_name',
                'serial_number',
                'condition',
                'image_path',
                'notes'
            ]);
        });
    }
};
