<?php

namespace Database\Seeders;

use App\Models\Permission;
use App\Models\Role;
use App\Models\User;
use Illuminate\Database\Seeder;
use Illuminate\Support\Str;

class RolesAndPermissionsSeeder extends Seeder
{
    /**
     * Run the database seeds.
     */
    public function run(): void
    {
        // إنشاء صلاحيات المستخدمين
        $userPermissions = [
            ['name' => 'عرض المستخدمين', 'slug' => 'users.view', 'group' => 'users', 'description' => 'يمكن للمستخدم عرض قائمة المستخدمين'],
            ['name' => 'إنشاء مستخدم', 'slug' => 'users.create', 'group' => 'users', 'description' => 'يمكن للمستخدم إنشاء مستخدم جديد'],
            ['name' => 'تعديل مستخدم', 'slug' => 'users.edit', 'group' => 'users', 'description' => 'يمكن للمستخدم تعديل بيانات المستخدمين'],
            ['name' => 'حذف مستخدم', 'slug' => 'users.delete', 'group' => 'users', 'description' => 'يمكن للمستخدم حذف مستخدم من النظام'],
        ];

        // إنشاء صلاحيات الأدوار
        $rolePermissions = [
            ['name' => 'عرض الأدوار', 'slug' => 'roles.view', 'group' => 'roles', 'description' => 'يمكن للمستخدم عرض قائمة الأدوار'],
            ['name' => 'إنشاء دور', 'slug' => 'roles.create', 'group' => 'roles', 'description' => 'يمكن للمستخدم إنشاء دور جديد'],
            ['name' => 'تعديل دور', 'slug' => 'roles.edit', 'group' => 'roles', 'description' => 'يمكن للمستخدم تعديل بيانات الأدوار'],
            ['name' => 'حذف دور', 'slug' => 'roles.delete', 'group' => 'roles', 'description' => 'يمكن للمستخدم حذف دور من النظام'],
        ];

        // إنشاء صلاحيات الصلاحيات
        $permissionPermissions = [
            ['name' => 'عرض الصلاحيات', 'slug' => 'permissions.view', 'group' => 'permissions', 'description' => 'يمكن للمستخدم عرض قائمة الصلاحيات'],
            ['name' => 'إنشاء صلاحية', 'slug' => 'permissions.create', 'group' => 'permissions', 'description' => 'يمكن للمستخدم إنشاء صلاحية جديدة'],
            ['name' => 'تعديل صلاحية', 'slug' => 'permissions.edit', 'group' => 'permissions', 'description' => 'يمكن للمستخدم تعديل بيانات الصلاحيات'],
            ['name' => 'حذف صلاحية', 'slug' => 'permissions.delete', 'group' => 'permissions', 'description' => 'يمكن للمستخدم حذف صلاحية من النظام'],
        ];

        // إنشاء صلاحيات الإدارات
        $departmentPermissions = [
            ['name' => 'عرض الإدارات', 'slug' => 'departments.view', 'group' => 'departments', 'description' => 'يمكن للمستخدم عرض قائمة الإدارات'],
            ['name' => 'إنشاء إدارة', 'slug' => 'departments.create', 'group' => 'departments', 'description' => 'يمكن للمستخدم إنشاء إدارة جديدة'],
            ['name' => 'تعديل إدارة', 'slug' => 'departments.edit', 'group' => 'departments', 'description' => 'يمكن للمستخدم تعديل بيانات الإدارات'],
            ['name' => 'حذف إدارة', 'slug' => 'departments.delete', 'group' => 'departments', 'description' => 'يمكن للمستخدم حذف إدارة من النظام'],
        ];

        // إنشاء صلاحيات الأصول
        $assetPermissions = [
            ['name' => 'عرض قائمة الأصول', 'slug' => 'view-any-assets', 'group' => 'assets', 'description' => 'يمكن للمستخدم عرض قائمة جميع الأصول'],
            ['name' => 'عرض تفاصيل أصل', 'slug' => 'view-assets', 'group' => 'assets', 'description' => 'يمكن للمستخدم عرض تفاصيل أصل معين'],
            ['name' => 'إنشاء أصل', 'slug' => 'create-assets', 'group' => 'assets', 'description' => 'يمكن للمستخدم إنشاء أصل جديد'],
            ['name' => 'تعديل أصل', 'slug' => 'update-assets', 'group' => 'assets', 'description' => 'يمكن للمستخدم تعديل بيانات الأصول'],
            ['name' => 'حذف أصل', 'slug' => 'delete-assets', 'group' => 'assets', 'description' => 'يمكن للمستخدم حذف أصل من النظام'],
        ];

        // إنشاء صلاحيات المدارس
        $schoolPermissions = [
            ['name' => 'عرض قائمة المدارس', 'slug' => 'view-any-schools', 'group' => 'schools', 'description' => 'يمكن للمستخدم عرض قائمة جميع المدارس'],
            ['name' => 'عرض تفاصيل مدرسة', 'slug' => 'view-schools', 'group' => 'schools', 'description' => 'يمكن للمستخدم عرض تفاصيل مدرسة معينة'],
            ['name' => 'إنشاء مدرسة', 'slug' => 'create-schools', 'group' => 'schools', 'description' => 'يمكن للمستخدم إنشاء مدرسة جديدة'],
            ['name' => 'تعديل مدرسة', 'slug' => 'update-schools', 'group' => 'schools', 'description' => 'يمكن للمستخدم تعديل بيانات المدارس'],
            ['name' => 'حذف مدرسة', 'slug' => 'delete-schools', 'group' => 'schools', 'description' => 'يمكن للمستخدم حذف مدرسة من النظام'],
        ];

        // إنشاء صلاحيات فئات الأصول
        $assetCategoryPermissions = [
            ['name' => 'عرض قائمة فئات الأصول', 'slug' => 'view-any-asset-categories', 'group' => 'asset-categories', 'description' => 'يمكن للمستخدم عرض قائمة جميع فئات الأصول'],
            ['name' => 'عرض تفاصيل فئة أصل', 'slug' => 'view-asset-categories', 'group' => 'asset-categories', 'description' => 'يمكن للمستخدم عرض تفاصيل فئة أصل معينة'],
            ['name' => 'إنشاء فئة أصل', 'slug' => 'create-asset-categories', 'group' => 'asset-categories', 'description' => 'يمكن للمستخدم إنشاء فئة أصل جديدة'],
            ['name' => 'تعديل فئة أصل', 'slug' => 'update-asset-categories', 'group' => 'asset-categories', 'description' => 'يمكن للمستخدم تعديل بيانات فئات الأصول'],
            ['name' => 'حذف فئة أصل', 'slug' => 'delete-asset-categories', 'group' => 'asset-categories', 'description' => 'يمكن للمستخدم حذف فئة أصل من النظام'],
        ];

        // إنشاء صلاحيات ماركات الأصول
        $assetBrandPermissions = [
            ['name' => 'عرض قائمة ماركات الأصول', 'slug' => 'view-any-asset-brands', 'group' => 'asset-brands', 'description' => 'يمكن للمستخدم عرض قائمة جميع ماركات الأصول'],
            ['name' => 'عرض تفاصيل ماركة أصل', 'slug' => 'view-asset-brands', 'group' => 'asset-brands', 'description' => 'يمكن للمستخدم عرض تفاصيل ماركة أصل معينة'],
            ['name' => 'إنشاء ماركة أصل', 'slug' => 'create-asset-brands', 'group' => 'asset-brands', 'description' => 'يمكن للمستخدم إنشاء ماركة أصل جديدة'],
            ['name' => 'تعديل ماركة أصل', 'slug' => 'update-asset-brands', 'group' => 'asset-brands', 'description' => 'يمكن للمستخدم تعديل بيانات ماركات الأصول'],
            ['name' => 'حذف ماركة أصل', 'slug' => 'delete-asset-brands', 'group' => 'asset-brands', 'description' => 'يمكن للمستخدم حذف ماركة أصل من النظام'],
        ];

        // دمج جميع الصلاحيات
        $allPermissions = array_merge(
            $userPermissions,
            $rolePermissions,
            $permissionPermissions,
            $departmentPermissions,
            $assetPermissions,
            $schoolPermissions,
            $assetCategoryPermissions,
            $assetBrandPermissions
        );

        // إنشاء الصلاحيات في قاعدة البيانات أو تحديثها إذا كانت موجودة
        foreach ($allPermissions as $permissionData) {
            Permission::updateOrCreate(
                ['slug' => $permissionData['slug']], // Check by slug
                $permissionData // Data to create or update with
            );
        }

        // إنشاء الأدوار أو تحديثها
        $adminRole = Role::updateOrCreate(
            ['slug' => 'admin'],
            [
                'name' => 'مدير النظام',
                'description' => 'لديه جميع الصلاحيات في النظام',
                'is_active' => true,
            ]
        );

        $editorRole = Role::updateOrCreate(
            ['slug' => 'editor'],
            [
                'name' => 'محرر',
                'description' => 'يمكنه تعديل المحتوى ولكن لا يمكنه إدارة المستخدمين والأدوار',
                'is_active' => true,
            ]
        );

        $userRole = Role::updateOrCreate(
            ['slug' => 'user'],
            [
                'name' => 'مستخدم',
                'description' => 'مستخدم عادي بصلاحيات محدودة',
                'is_active' => true,
            ]
        );

        // إسناد جميع الصلاحيات للمدير (إضافة الصلاحيات الجديدة فقط)
        $adminRole->permissions()->syncWithoutDetaching(Permission::pluck('id'));

        // إسناد صلاحيات محددة للمحرر (إضافة الصلاحيات الجديدة فقط)
        $editorPermissions = Permission::whereIn('slug', [
            'users.view',
            'roles.view',
            'permissions.view',
            // Add any specific department permissions for editor if needed
            'departments.view',
        ])->pluck('id');
        $editorRole->permissions()->syncWithoutDetaching($editorPermissions);

        // إنشاء مستخدم مدير افتراضي أو تحديثه إذا كان موجوداً
        $admin = User::updateOrCreate(
            ['email' => 'momari.c@tetco.sa'], // Check by email
            [
                'name' => 'مدير النظام',
                'password' => bcrypt('Ads!@123123'), // Update password if needed
                'email_verified_at' => now(), // Ensure it's verified
                'remember_token' => Str::random(10), // Refresh token
            ]
        );

        // إسناد دور المدير للمستخدم المدير (مع التأكد من عدم وجود تكرار)
        $admin->roles()->syncWithoutDetaching([$adminRole->id]);
    }
}
